==> derby-0.0.3.tgz: package/lib/View.js <==
var View, addNameToData, dataValue, elementParser, extend, extractPlaceholder, htmlEscape, htmlParser, modelPath, modelText, parse, parseString, quoteAttr, reduceStack, renderer, trim, trimInner;
htmlParser = require('./htmlParser');
elementParser = require('./elementParser');
View = module.exports = function() {
  var self;
  self = this;
  this._views = {};
  this._loadFuncs = '';
  this._idCount = 0;
  this._uniqueId = function() {
    return '$' + (self._idCount++).toString(36);
  };
};
View.prototype = {
  get: function(viewName, ctx, parentCtx) {
    var i, item, parentView, type, view;
    if (!(view = this._views[viewName])) {
      if (~(i = viewName.indexOf('$'))) {
        parentView = viewName.substr(1, i - 1);
        if (!this._views[parentView]) {
          throw "Can't find view: " + parentView;
        }
        this.get(parentView);
        return this.get(viewName, ctx, parentCtx);
      }
      return '';
    }
    if (parentCtx || (ctx != null)) {
      if (Array.isArray(ctx)) {
        return ((function() {
          var _i, _len, _results;
          _results = [];
          for (_i = 0, _len = ctx.length; _i < _len; _i++) {
            item = ctx[_i];
            _results.push(view(extend(parentCtx, item)));
          }
          return _results;
        })()).join('');
      }
      ctx = (type = viewName.charAt(0)) === '^' ? !ctx : type === '#' ? !!ctx : ctx;
      if (ctx === false) {
        return '';
      }
    }
    return view(extend(parentCtx, ctx));
  },
  preLoad: function(fn) {
    return this._loadFuncs += "(" + fn + ")();";
  },
  make: function(name, template, data) {
    var fn, render, self;
    if (data == null) {
      data = {};
    }
    self = this;
    render = function(ctx) {
      if (typeof template === 'string' && !~template.indexOf('{{')) {
        template = trim(template);
        render = function() {
          return template;
        };
      } else {
        render = parse(self, name, template, data);
      }
      return render(ctx);
    };
    fn = function(ctx) {
      return render(extend(data, ctx));
    };
    return this._register(name, fn, data.before, data.after);
  },
  _register: function(name, fn, before, after) {
    return this._views[name] = before ? after ? function(ctx) {
      var rendered;
      before();
      setTimeout(after, 0);
      return rendered = fn(ctx);
    } : function(ctx) {
      before();
      return fn(ctx);
    } : after ? function(ctx) {
      var rendered;
      setTimeout(after, 0);
      return rendered = fn(ctx);
    } : fn;
  }
};
extend = function(parent, obj) {
  var key, out;
  if (typeof parent !== 'object') {
    return obj;
  }
  if (!obj) {
    return parent;
  }
  out = Object.create(parent);
  for (key in obj) {
    out[key] = obj[key];
  }
  return out;
};
View.htmlEscape = htmlEscape = function(s) {
  if (s == null) {
    return '';
  } else {
    return s.replace(/[&<>]/g, function(s) {
      switch (s) {
        case '&':
          return '&amp;';
        case '<':
          return '&lt;';
        case '>':
          return '&gt;';
        default:
          return s;
      }
    });
  }
};
quoteAttr = function(s) {
  if (s == null || s === '') {
    return '""';
  }
  s = s.toString().replace(/"/g, '&quot;');
  if (/[ =]/.test(s)) {
    return '"' + s + '"';
  } else {
    return s;
  }
};
trim = function(s) {
  if (s) {
    return s.replace(/(?:^|\n)\s*/g, '');
  } else {
    return '';
  }
};
trimInner = function(s) {
  if (s) {
    return s.replace(/\n\s*/g, '');
  } else {
    return '';
  }
};
extractPlaceholder = function(text) {
  var content, match;
  match = /^([^\{]*)(\{{2,3})([^\}]+)\}{2,3}([\s\S]*)/.exec(text);
  if (!match) {
    return;
  }
  content = /^([#^//]?) *([^ >]*)(?: *> *(.*))?/.exec(match[3]);
  return {
    pre: trimInner(match[1]),
    escaped: match[2] === '{{',
    type: content[1],
    name: content[2],
    partial: content[3],
    post: trimInner(match[4])
  };
};
addNameToData = function(data, name) {
  if (name && !(name in data)) {
    return data[name] = {
      model: name
    };
  }
};
modelPath = function(data, name) {
  var datum, path;
  datum = data[name];
  if (!(datum && (path = datum.model))) {
    return null;
  }
  return path.replace(/\(([^)]+)\)/g, function(match, name) {
    return data[name];
  });
};
dataValue = function(data, name, model) {
  var path, value;
  if (path = modelPath(data, name)) {
    if ((value = model.get(path)) != null) {
      return value;
    } else {
      return model[path];
    }
  } else {
    return data[name];
  }
};
modelText = function(view, name, escaped, quote) {
  return function(data, model) {
    var text;
    text = dataValue(data, name, model);
    text = text != null ? text.toString() : '';
    if (escaped) {
      text = htmlEscape(text);
    }
    if (quote) {
      text = quoteAttr(text);
    }
    return text;
  };
};
reduceStack = function(stack) {
  var html, i, item, key, pushValue, value, _i, _len, _ref;
  html = [''];
  i = 0;
  for (_i = 0, _len = stack.length; _i < _len; _i++) {
    item = stack[_i];
    pushValue = function(value, quote) {
      if (value && value.call) {
        return i = html.push(value, '') - 1;
      } else {
        return html[i] += quote ? quoteAttr(value) : value;
      }
    };
    switch (item[0]) {
      case 'start':
        html[i] += '<' + item[1];
        _ref = item[2];
        for (key in _ref) {
          value = _ref[key];
          html[i] += ' ' + key + '=';
          pushValue(value, true);
        }
        html[i] += '>';
        break;
      case 'chars':
        pushValue(item[1]);
        break;
      case 'end':
        html[i] += '</' + item[1] + '>';
    }
  }
  return html;
};
renderer = function(view, items, events) {
  return function(data) {
    var event, html, item, model, modelEvents, _i, _len;
    model = view.model;
    modelEvents = model.__events;
    html = ((function() {
      var _i, _len, _results;
      _results = [];
      for (_i = 0, _len = items.length; _i < _len; _i++) {
        item = items[_i];
        _results.push((item.call ? item(data, model) : item));
      }
      return _results;
    })()).join('');
    for (_i = 0, _len = events.length; _i < _len; _i++) {
      event = events[_i];
      event(data, modelEvents);
    }
    return html;
  };
};
parse = function(view, viewName, template, data) {
  var block, chars, elements, events, partialCount, partialName, popped, queue, queues, stack, uniqueId, _fn, _i, _len;
  if (viewName === 'Title') {
    return parseString(view, viewName, template, data);
  }
  queues = [
    {
      stack: stack = [],
      events: events = []
    }
  ];
  popped = [];
  block = null;
  uniqueId = view._uniqueId;
  partialCount = 0;
  partialName = function() {
    return viewName + '$' + partialCount++;
  };
  elements = elementParser(view, events);
  htmlParser.parse(template, {
    start: function(tag, tagName, attrs) {
      var escaped, key, match, method, name, value;
      for (key in attrs) {
        value = attrs[key];
        if (match = extractPlaceholder(value)) {
          name = match.name, escaped = match.escaped;
          addNameToData(data, name);
          if (attrs.id === void 0) {
            attrs.id = function() {
              return attrs._id = uniqueId();
            };
          }
          method = tagName in elements ? elements[tagName](key, attrs, name) : 'attr';
          events.push(function(data, modelEvents) {
            var path;
            path = modelPath(data, name);
            if (path) {
              return modelEvents.bind(path, [attrs._id || attrs.id, method, key]);
            }
          });
          attrs[key] = modelText(view, name, escaped, true);
        }
      }
      return stack.push(['start', tagName, attrs]);
    },
    chars: chars = function(text) {
      var addEvent, attrs, autoClosed, endBlock, escaped, i, last, lastAutoClosed, lastPartial, match, name, partial, partialText, post, pre, startBlock, type, wrap, _ref;
      if (match = extractPlaceholder(text)) {
        pre = match.pre, post = match.post, name = match.name, escaped = match.escaped, type = match.type, partial = match.partial;
        addNameToData(data, name);
        text = '';
      }
      if (pre) {
        stack.push(['chars', pre]);
      }
      if (block && (type === '/' || (autoClosed = type === '^' && !name && block.type === '#'))) {
        name = block.name, partial = block.partial, lastPartial = block.lastPartial, lastAutoClosed = block.lastAutoClosed;
        popped.push(queues.pop());
        _ref = queues[queues.length - 1], stack = _ref.stack, events = _ref.events, block = _ref.block;
      }
      if (startBlock = type === '#' || type === '^') {
        lastPartial = partial;
        partial = type + partialName();
        block = {
          type: type,
          name: name,
          partial: partial,
          lastPartial: lastPartial,
          lastAutoClosed: autoClosed
        };
      }
      if (partial) {
        partialText = function(data, model) {
          return view.get(partial, dataValue(data, name, model), data);
        };
      }
      if (name && !startBlock) {
        endBlock = type === '/';
        i = stack.length - (endBlock ? (lastAutoClosed ? 3 : 2) : 1);
        last = stack[i];
        if (wrap = pre || post || !(last && last[0] === 'start')) {
          last = ['start', 'ins', {}];
          if (endBlock) {
            stack.splice(i + 1, 0, last);
          } else {
            stack.push(last);
          }
        }
        attrs = last[2];
        if (attrs.id === void 0) {
          attrs.id = function() {
            return attrs._id = uniqueId();
          };
        }
        addEvent = function(partial, append) {
          return events.push(function(data, modelEvents) {
            var domMethod, params, path;
            if (!(path = modelPath(data, name))) {
              return;
            }
            if (partial) {
              escaped = false;
            }
            domMethod = append ? 'appendHtml' : 'html';
            params = [attrs._id || attrs.id, domMethod, +escaped];
            if (partial) {
              params[3] = partial;
            }
            return modelEvents.bind(path, params);
          });
        };
        addEvent(partial);
        if (lastAutoClosed) {
          addEvent(lastPartial, true);
        }
        if (!endBlock) {
          text = partialText || modelText(view, name, escaped);
        }
      } else {
        text = partialText || text;
      }
      if (text) {
        stack.push(['chars', text]);
      }
      if (wrap) {
        stack.push(['end', 'ins']);
      }
      if (startBlock) {
        queues.push({
          stack: stack = [],
          events: events = [],
          viewName: partial,
          block: block
        });
      }
      if (post) {
        return chars(post);
      }
    },
    end: function(tag, tagName) {
      return stack.push(['end', tagName]);
    }
  });
  _fn = function(queue) {
    var render;
    render = renderer(view, reduceStack(queue.stack), queue.events);
    return view._register(queue.viewName, function(ctx) {
      return render(extend(data, ctx));
    });
  };
  for (_i = 0, _len = popped.length; _i < _len; _i++) {
    queue = popped[_i];
    _fn(queue);
  }
  return renderer(view, reduceStack(stack), events);
};
parseString = function(view, viewName, template, data) {
  var events, items, name, params, post, pre, _ref;
  items = [];
  events = [];
  post = template;
  while (post) {
    _ref = extractPlaceholder(post), name = _ref.name, pre = _ref.pre, post = _ref.post;
    addNameToData(data, name);
    if (pre) {
      items.push(pre);
    }
    items.push(modelText(view, name));
    if (viewName === 'Title') {
      params = ['$doc', 'prop', 'title', 'Title'];
    }
    (function(name) {
      if (params) {
        return events.push(function(data, modelEvents) {
          var path;
          if (!(path = modelPath(data, name))) {
            return;
          }
          return modelEvents.bind(path, params.slice());
        });
      }
    })(name);
  }
  return renderer(view, items, events);
};

==> derby-0.0.5.tgz: package/lib/View.js <==
var View, addId, addNameToData, dataValue, elementParser, extend, extractPlaceholder, htmlEscape, htmlParser, modelPath, modelText, parse, parseString, quoteAttr, reduceStack, renderer, trim, trimInner;
htmlParser = require('./htmlParser');
elementParser = require('./elementParser');
View = module.exports = function() {
  var self;
  self = this;
  this._views = {};
  this._loadFuncs = '';
  this._idCount = 0;
  this._uniqueId = function() {
    return '$' + (self._idCount++).toString(36);
  };
};
View.prototype = {
  get: function(viewName, ctx, parentCtx) {
    var i, item, parentView, type, view;
    if (!(view = this._views[viewName])) {
      if (~(i = viewName.indexOf('$'))) {
        parentView = viewName.substr(1, i - 1);
        if (!this._views[parentView]) {
          throw "Can't find view: " + parentView;
        }
        this.get(parentView);
        return this.get(viewName, ctx, parentCtx);
      }
      return '';
    }
    if (parentCtx || (ctx != null)) {
      if (Array.isArray(ctx)) {
        return ((function() {
          var _i, _len, _results;
          _results = [];
          for (_i = 0, _len = ctx.length; _i < _len; _i++) {
            item = ctx[_i];
            _results.push(view(extend(parentCtx, item)));
          }
          return _results;
        })()).join('');
      }
      ctx = (type = viewName.charAt(0)) === '^' ? !ctx : type === '#' ? !!ctx : ctx;
      if (ctx === false) {
        return '';
      }
    }
    return view(extend(parentCtx, ctx));
  },
  preLoad: function(fn) {
    return this._loadFuncs += "(" + fn + ")();";
  },
  make: function(name, template, data) {
    var fn, render, self;
    if (data == null) {
      data = {};
    }
    self = this;
    render = function(ctx) {
      if (typeof template === 'string' && !~template.indexOf('{{')) {
        template = trim(template);
        render = function() {
          return template;
        };
      } else {
        render = parse(self, name, template, data);
      }
      return render(ctx);
    };
    fn = function(ctx) {
      return render(extend(data, ctx));
    };
    return this._register(name, fn, data.Before, data.After);
  },
  _register: function(name, fn, before, after) {
    return this._views[name] = before ? after ? function(ctx) {
      var rendered;
      before();
      setTimeout(after, 0);
      return rendered = fn(ctx);
    } : function(ctx) {
      before();
      return fn(ctx);
    } : after ? function(ctx) {
      var rendered;
      setTimeout(after, 0);
      return rendered = fn(ctx);
    } : fn;
  }
};
extend = function(parent, obj) {
  var key, out;
  if (typeof parent !== 'object') {
    return obj;
  }
  if (!obj) {
    return parent;
  }
  out = Object.create(parent);
  for (key in obj) {
    out[key] = obj[key];
  }
  return out;
};
addId = function(attrs, uniqueId) {
  if (attrs.id === void 0) {
    return attrs.id = function() {
      return attrs._id = uniqueId();
    };
  }
};
View.htmlEscape = htmlEscape = function(s) {
  if (s == null) {
    return '';
  } else {
    return s.replace(/[&<>]/g, function(s) {
      switch (s) {
        case '&':
          return '&amp;';
        case '<':
          return '&lt;';
        case '>':
          return '&gt;';
        default:
          return s;
      }
    });
  }
};
quoteAttr = function(s) {
  if (s == null || s === '') {
    return '""';
  }
  s = s.toString().replace(/"/g, '&quot;');
  if (/[ =]/.test(s)) {
    return '"' + s + '"';
  } else {
    return s;
  }
};
trim = function(s) {
  if (s) {
    return s.replace(/(?:^|\n)\s*/g, '');
  } else {
    return '';
  }
};
trimInner = function(s) {
  if (s) {
    return s.replace(/\n\s*/g, '');
  } else {
    return '';
  }
};
extractPlaceholder = function(text) {
  var content, match;
  match = /^([^\{]*)(\{{2,3})([^\}]+)\}{2,3}([\s\S]*)/.exec(text);
  if (!match) {
    return;
  }
  content = /^([#^//]?) *([^ >]*)(?: *> *(.*))?/.exec(match[3]);
  return {
    pre: trimInner(match[1]),
    escaped: match[2] === '{{',
    type: content[1],
    name: content[2],
    partial: content[3],
    post: trimInner(match[4])
  };
};
addNameToData = function(data, name) {
  if (name && !(name in data)) {
    return data[name] = {
      model: name
    };
  }
};
modelPath = function(data, name) {
  var datum, path;
  datum = data[name];
  if (!(datum && (path = datum.model))) {
    return null;
  }
  return path.replace(/\(([^)]+)\)/g, function(match, name) {
    return data[name];
  });
};
dataValue = function(data, name, model) {
  var path, value;
  if (path = modelPath(data, name)) {
    if ((value = model.get(path)) != null) {
      return value;
    } else {
      return model[path];
    }
  } else {
    return data[name];
  }
};
modelText = function(view, name, escaped, quote) {
  return function(data, model) {
    var text;
    text = dataValue(data, name, model);
    text = text != null ? text.toString() : '';
    if (escaped) {
      text = htmlEscape(text);
    }
    if (quote) {
      text = quoteAttr(text);
    }
    return text;
  };
};
reduceStack = function(stack) {
  var html, i, item, key, pushValue, value, _i, _len, _ref;
  html = [''];
  i = 0;
  for (_i = 0, _len = stack.length; _i < _len; _i++) {
    item = stack[_i];
    pushValue = function(value, quote) {
      if (value && value.call) {
        return i = html.push(value, '') - 1;
      } else {
        return html[i] += quote ? quoteAttr(value) : value;
      }
    };
    switch (item[0]) {
      case 'start':
        html[i] += '<' + item[1];
        _ref = item[2];
        for (key in _ref) {
          value = _ref[key];
          html[i] += ' ' + key + '=';
          pushValue(value, true);
        }
        html[i] += '>';
        break;
      case 'chars':
        pushValue(item[1]);
        break;
      case 'end':
        html[i] += '</' + item[1] + '>';
    }
  }
  return html;
};
renderer = function(view, items, events) {
  return function(data) {
    var domEvents, event, html, item, model, modelEvents, _i, _len;
    model = view.model;
    modelEvents = model.__events;
    domEvents = view.dom.events;
    html = ((function() {
      var _i, _len, _results;
      _results = [];
      for (_i = 0, _len = items.length; _i < _len; _i++) {
        item = items[_i];
        _results.push((item.call ? item(data, model) : item));
      }
      return _results;
    })()).join('');
    for (_i = 0, _len = events.length; _i < _len; _i++) {
      event = events[_i];
      event(data, modelEvents, domEvents);
    }
    return html;
  };
};
parse = function(view, viewName, template, data) {
  var block, chars, elementBind, elementPlaceholder, events, partialCount, partialName, popped, queue, queues, stack, uniqueId, _fn, _i, _len, _ref;
  if (viewName === 'Title') {
    return parseString(view, viewName, template, data);
  }
  queues = [
    {
      stack: stack = [],
      events: events = []
    }
  ];
  popped = [];
  block = null;
  uniqueId = view._uniqueId;
  partialCount = 0;
  partialName = function() {
    return viewName + '$' + partialCount++;
  };
  _ref = elementParser(view, events), elementPlaceholder = _ref.elementPlaceholder, elementBind = _ref.elementBind;
  htmlParser.parse(template, {
    start: function(tag, tagName, attrs) {
      var attr, escaped, fn, match, method, name, value;
      for (attr in attrs) {
        value = attrs[attr];
        if (match = extractPlaceholder(value)) {
          name = match.name, escaped = match.escaped;
          addNameToData(data, name);
          addId(attrs, uniqueId);
          method = tagName in elementPlaceholder ? elementPlaceholder[tagName](attr, attrs, name) : 'attr';
          events.push(function(data, modelEvents) {
            var path;
            path = modelPath(data, name);
            if (path) {
              return modelEvents.bind(path, [attrs._id || attrs.id, method, attr]);
            }
          });
          attrs[attr] = modelText(view, name, escaped, true);
        }
        if (attr === 'bind') {
          if (!(match = /^\s*([^:\s]+)\s*:\s*([^\s]+)/.exec(value))) {
            break;
          }
          name = match[1];
          fn = match[2];
          delete attrs[attr];
          addId(attrs, uniqueId);
          if (tagName in elementBind) {
            elementBind[tagName](attrs, name);
          }
          events.push(function(data, modelEvents, domEvents) {
            return domEvents.bind(name, [fn, attrs._id || attrs.id]);
          });
        }
      }
      return stack.push(['start', tagName, attrs]);
    },
    chars: chars = function(text) {
      var addEvent, attrs, autoClosed, endBlock, escaped, i, last, lastAutoClosed, lastPartial, match, name, partial, partialText, post, pre, startBlock, type, wrap, _ref2;
      if (match = extractPlaceholder(text)) {
        pre = match.pre, post = match.post, name = match.name, escaped = match.escaped, type = match.type, partial = match.partial;
        addNameToData(data, name);
        text = '';
      } else {
        text = trimInner(text);
      }
      if (pre) {
        stack.push(['chars', pre]);
      }
      if (block && (type === '/' || (autoClosed = type === '^' && !name && block.type === '#'))) {
        name = block.name, partial = block.partial, lastPartial = block.lastPartial, lastAutoClosed = block.lastAutoClosed;
        popped.push(queues.pop());
        _ref2 = queues[queues.length - 1], stack = _ref2.stack, events = _ref2.events, block = _ref2.block;
      }
      if (startBlock = type === '#' || type === '^') {
        lastPartial = partial;
        partial = type + partialName();
        block = {
          type: type,
          name: name,
          partial: partial,
          lastPartial: lastPartial,
          lastAutoClosed: autoClosed
        };
      }
      if (partial) {
        partialText = function(data, model) {
          return view.get(partial, dataValue(data, name, model), data);
        };
      }
      if (name && !startBlock) {
        endBlock = type === '/';
        i = stack.length - (endBlock ? (lastAutoClosed ? 3 : 2) : 1);
        last = stack[i];
        if (wrap = pre || post || !(last && last[0] === 'start')) {
          last = ['start', 'ins', {}];
          if (endBlock) {
            stack.splice(i + 1, 0, last);
          } else {
            stack.push(last);
          }
        }
        attrs = last[2];
        addId(attrs, uniqueId);
        addEvent = function(partial, append) {
          return events.push(function(data, modelEvents) {
            var domMethod, params, path;
            if (!(path = modelPath(data, name))) {
              return;
            }
            if (partial) {
              escaped = false;
            }
            domMethod = append ? 'appendHtml' : 'html';
            params = [attrs._id || attrs.id, domMethod, +escaped];
            if (partial) {
              params[3] = partial;
            }
            return modelEvents.bind(path, params);
          });
        };
        addEvent(partial);
        if (lastAutoClosed) {
          addEvent(lastPartial, true);
        }
        if (!endBlock) {
          text = partialText || modelText(view, name, escaped);
        }
      } else {
        text = partialText || text;
      }
      if (text) {
        stack.push(['chars', text]);
      }
      if (wrap) {
        stack.push(['end', 'ins']);
      }
      if (startBlock) {
        queues.push({
          stack: stack = [],
          events: events = [],
          viewName: partial,
          block: block
        });
      }
      if (post) {
        return chars(post);
      }
    },
    end: function(tag, tagName) {
      return stack.push(['end', tagName]);
    }
  });
  _fn = function(queue) {
    var render;
    render = renderer(view, reduceStack(queue.stack), queue.events);
    return view._register(queue.viewName, function(ctx) {
      return render(extend(data, ctx));
    });
  };
  for (_i = 0, _len = popped.length; _i < _len; _i++) {
    queue = popped[_i];
    _fn(queue);
  }
  return renderer(view, reduceStack(stack), events);
};
parseString = function(view, viewName, template, data) {
  var events, items, name, params, post, pre, _ref;
  items = [];
  events = [];
  post = template;
  while (post) {
    _ref = extractPlaceholder(post), name = _ref.name, pre = _ref.pre, post = _ref.post;
    addNameToData(data, name);
    if (pre) {
      items.push(pre);
    }
    items.push(modelText(view, name));
    if (viewName === 'Title') {
      params = ['$doc', 'prop', 'title', 'Title'];
    }
    (function(name) {
      if (params) {
        return events.push(function(data, modelEvents) {
          var path;
          if (!(path = modelPath(data, name))) {
            return;
          }
          return modelEvents.bind(path, params.slice());
        });
      }
    })(name);
  }
  return renderer(view, items, events);
};
